package info1.ships;

import java.util.*;

/**
 * Classe définissant une flotte de navires
 */

public class NavyFleet implements INavyFleet {

    private List<IShip> flotte;

    /*
    Cette classe a pour but de créer une flotte de ship accompagné de toutes ses méthodes.
     */

    /**
     * NB : LA SIGNATURE DU CONSTRUCTEUR DOIT ETRE RESPECTEE
     *
     * Construit une nouvelle flotte
     */
    //---------------Constructeur NavyFleet---------------------
    public NavyFleet() {
        flotte = new ArrayList<>();
    }

    @Override
    public int remainingSize() {   //Fait par Corentin
        int compt=0;
        for(IShip courantIship : flotte){           //For each en additionnant la taille de case de chaque navire
            compt= compt+courantIship.getSize();
        }
        return  20-compt;               //Retourne le nombre de cases restante ( Il y a 20 cases initialement
    }

    @Override
    public boolean isComplete() {   //Fait par Corentin
        if(remainingSize()==0){     //Appel de la fonction remainingSize pour savor s'il reste de la place, si =0 alors c'est complet
            return true;
        }
        return false;
    }


    @Override
    public int add(IShip IShip) {   //Fait par Corentin
        List<ICoord> lisIcoordIship = IShip.getCoords(); //Ajout des coords du bateaux dans une liste de coord

        //------------Si la taille du bateau dépasse la place restante de a flotte-----------------
        if(IShip.getSize() > remainingSize()) {
            return -2;
        }
        //---------------------------Si le bateau est déjà dans la flotte--------------------------
        for(IShip courantIship : flotte){           //For each pour voir les bateau de la flotte actuel
            if(courantIship.equals(IShip)){         //Si le bateau entrant est déjà dans la flotte
                return -1;
            }
        }
        //--------------Cas du bateau avec des coords déjà utilisé dans la flotte--------------------
        for(IShip bateauCourant : flotte) {         //For each pour voir les bateau de la flotte actuel
            //----Ajout des coords des bateaux de la flotte dans une liste de ICoord----
            List<ICoord> listIcoordBateauCourant = bateauCourant.getCoords();
            for (ICoord coordonneeCourant : listIcoordBateauCourant) {      //For each pour avoir les coordonnes de la liste de coords de la flotte
                for(ICoord coordonneeIship : lisIcoordIship){               //For each pour avoir les coords du bateau en entré
                    if(coordonneeIship.equals(coordonneeCourant)){          //Si les coords du bateau courant sont égale à celle de la flotte
                        return -3;
                    }
                }
            }
        }
        //------------------------------Cas de l'ajout du bateau à la flotte------------------------
        flotte.add(IShip);
        return 0;
    }

    @Override
    public List<IShip> getShips() {     //Fait par Helori
        List<IShip> trie = new ArrayList<>(flotte);   //Création d'une copie de la liste actuelle
        trie.sort(null);      //Appel de la méthode .sort(null)
        return trie;                //Retourne la liste trié
    }

    @Override
    public Set<IShip> getShips(ShipCategory shipCategory){          //Fait par Helori
        Set<IShip> listeCatégorie = new HashSet<>();                //Création de Set de catégorie(Collection sans doublons)
        for(IShip courantIShip : flotte){                           //For each pour regarder tous les éléments de la liste
            if(courantIShip.getCategory().equals(shipCategory)){    //Condition if pour vérifier si la catogérie est correcte
                listeCatégorie.add(courantIShip);                   //Ajout à Set
            }
        }
        return listeCatégorie;             //Retourne le Set
    }

    @Override
    public boolean isBelgianConfiguration() {   //Fait par Helori
        int cuirasse  = this.getShips(ShipCategory.BATTLESHIP).size();      //Récupération du nombre de cuirassé dans la flotte
        int croiseurs = this.getShips(ShipCategory.CRUISER).size();         //Récupération du nombre de croiseur dans la flotte
        int torpilleurs = this.getShips(ShipCategory.DESTROYER).size();     //Récupération du nombre de torpilleur dans la flotte
        int sousMarin = this.getShips(ShipCategory.SUBMARINE).size();       //Récupération du nombre de sous-marin dans la flotte
        if(cuirasse==1 && croiseurs==2 &&  torpilleurs==3 && sousMarin==4){ //Si la formation actuelle correspond à la formation Belge
            return true;
        }
        else{
            return false;
        }
    }

    @Override
    public boolean isFrenchConfiguration() {   //Fait par Corentin
        if(this.getShips(ShipCategory.AIRCRAFT_CARRIER).size()==1){         //Si le nombre actuelle de port-avion correspond à la formation Française
            if(this.getShips(ShipCategory.BATTLESHIP).size()==1){           //Si le nombre actuelle de cuirassé correspond à la formation Française
                if(this.getShips(ShipCategory.CRUISER).size()==2){          //Si le nombre actuelle de croiseur correspond à la formation Française
                    if(this.getShips(ShipCategory.DESTROYER).size()==2){    //Si le nombre actuelle de torpilleurs correspond à la formation Française
                        if(this.getShips(ShipCategory.SUBMARINE).size()==1){//Si le nombre actuelle de sous-marin correspond à la formation Française
                            return true;
                        }
                    }
                }
            }
        }
        return false;
    }


    @Override
    public String toString(){   //Fait par Helori  (Redéfinition de la méthode ToString
        String res="[";
        for(IShip courantIShip : flotte){                           //For each pour afficher les informations de chaque bateau de la flotte
            res+="{bateau:"+courantIShip.getName()+"coordonnees:";      //Affichage du nom du bateau de la flotte
            for(ICoord courantICoord : courantIShip.getCoords()){
                    res+="[{x:"+courantICoord.getX()+",y:"+courantICoord.getY()+"}]";       //Affichage des coordonnees x et y du bateau de la flotte
            }
        }
        res+="]";
        return res;
    }

    //Méthode pour renvoyer une liste des noms des bateaux config belge
    public String[] listeShipConfigBelge(){
        String listeRetour[] =  {"Cuirassé (4 cases)","Croiseur (3 cases)","Croiseur (3 cases)","Torpilleur (2 cases)", "Torpilleur (2 cases)", "Torpilleur (2 cases)", "Sous-marin (1 case)", "Sous-marin (1 case)", "Sous-marin (1 case)", "Sous-marin (1 case)"};
        return listeRetour;
    }

    //Méthode pour renvoyer une liste des noms des bateaux config française
    public String[] listeShipConfigFrance(){
        String listeRetour[] =  {"Cuirassé (4 cases)","Croiseur (3 cases)","Croiseur (3 cases)","Torpilleur (2 cases)", "Torpilleur (2 cases)", "Sous-marin (1 case)", "Porte-avion (5 cases)" };
        return listeRetour;
    }

}
