package info1.ships;

import java.util.ArrayList;
import java.util.List;
import java.util.Objects;

/**
 * une implémentation "abstraite" d'un bateau quelconque, de taille indéterminée
 */


public abstract class Ship implements IShip {

    private String nom;
    private String coordProue; //Avant du bateau
    private String coordPoupe; //Arrière du bateau


    /**
     * NB : LA SIGNATURE DU CONSTRUCTEUR DOIT ETRE RESPECTEE
     *
     * construit un bateau quelconque
     * @param name le nom du bateau
     * @param ayFront la coordonnée de la proue du bateau
     * @param ayBack la coordonnée de la poupe du bateau
     * @throws BadCoordException si l'une des coordonnées données ne définit pas une coordonnée alphanumérique correcte
     * @throws CoordsBadShipException si les coordonnées données ne permettent pas de définir un bateau correct :
     *  une ligne, une colonne, de la bonne taille, etc.
     */
    public Ship(String name, String ayFront, String ayBack) throws BadCoordException, CoordsBadShipException {
        if(!estCoordDonneeCorrect(ayFront) || !estCoordDonneeCorrect(ayBack)) //On lève une exception si les coordonnées de la poupe ou la proue sont incorrectes
            throw new BadCoordException();
        else{
            if(!estBateauCorrect(ayBack, ayFront)){ //Si les coordonnées de l'avant ne sont pas sur la même ligne / la même colonne
                throw new CoordsBadShipException(); //On lève l'exception indiquant que le bateau est incorrect
            }else{
                //Initialisation classique
                this.nom = name;
                this.coordProue = ayFront;
                this.coordPoupe = ayBack;
            }
        }
    }


    @Override
    public List<ICoord> getCoords() {
        List<ICoord> listeCoord = new ArrayList<>(); //On crée la liste à retourner à la fin
        ICoord proue = this.getFront(); //On récupère l'ICoord de la proue et de la poupe
        ICoord poupe = this.getBack();

        //Si la lettre (X) est égale (c'est à dire que le bateau est sur une même colonne et plusieurs lignes)
        if(proue.getAlphaX() == poupe.getAlphaX()){
            //Si proue.y > poupe.y on décrémente (bateau tourné vers le bas)
            if(proue.getY() > poupe.getY()){
                for (int i = 0; i < this.getSize(); i++){ //Pour i allant de 0 à la taille du bateau - 1
                    try {
                        ICoord ajout = new Coord(""+proue.getAlphaX()+""+ (proue.getY() - i)); //On crée une nouvelle coordonnée en décrémentant la coordY de la proue
                        listeCoord.add(ajout); //Ajout de cette coordonnée à la liste
                    } catch (BadCoordException e) {
                        e.printStackTrace();
                    }
                }
            }
            //Sinon on incrémente
            //Cela fonctionne pareil que ci-dessus, sauf qu'au lieu de décrémenter la coordY de la proue, on l'incrémente
            else{
                for (int i = 0; i < this.getSize(); i++){
                    try {
                        ICoord ajout = new Coord(""+proue.getAlphaX()+""+(proue.getY()+i));
                        listeCoord.add(ajout);
                    } catch (BadCoordException e) {
                        e.printStackTrace();
                    }
                }
            }


        }
        //Si les chiffres sont égaux, idem que les deux cas ci-dessus sauf qu'on incrémente / décrémente la lettre (coordX)
        else{
            //Si proue.x > poupe.x on décrémente
            if(proue.getX() > poupe.getX()){
                for(int i = 0; i < this.getSize(); i++){
                    try {
                        ICoord ajout = new Coord(""+(char)(proue.getAlphaX() - i)+""+proue.getY());
                        listeCoord.add(ajout);
                    } catch (BadCoordException e) {
                        e.printStackTrace();
                    }
                }
            }
            //Sinon on incrémente
            else{
                for(int i = 0; i < this.getSize(); i++){
                    try {
                        ICoord ajout = new Coord(""+(char)(proue.getAlphaX() + i) +""+proue.getY());
                        listeCoord.add(ajout);
                    } catch (BadCoordException e) {
                        e.printStackTrace();
                    }
                }
            }


        }
        return listeCoord;
    }

    @Override
    public ICoord getFront() {
        Coord proue = null; //Initialisation
        try {
            proue = new Coord(this.coordProue);
        } catch (BadCoordException e) {
            e.printStackTrace();
        }
        return proue;
    }

    @Override
    public ICoord getBack() {
        Coord poupe = null;
        try {
            poupe = new Coord(this.coordPoupe);
        } catch (BadCoordException e) {
            e.printStackTrace();
        }
        return poupe;
    }

    @Override
    public String getName() {
        return this.nom;
    }

    @Override
    public int getSize() {
        return Math.abs((getFront().getX()-getBack().getX())-(getFront().getY()-getBack().getY()))+1;  //Return la taille du navire
    }

    @Override
    public String toString() {
        return "Nom : "+this.nom + "\nCoordonnées de la proue : " + this.coordProue + "\nCoordonnées de la poupe" + this.coordPoupe + "Taille du navire : " + this.getSize();
    }


    @Override
    public boolean equals(Object o) {
        if (this == o) return true;
        if (o == null || getClass() != o.getClass()) return false;
        Ship ship = (Ship) o;
        return Objects.equals(nom, ship.nom) &&
                Objects.equals(coordProue, ship.coordProue) &&
                Objects.equals(coordPoupe, ship.coordPoupe);
    }

    @Override
    public int hashCode() {
        return Objects.hash(nom, coordProue, coordPoupe);
    }

    public boolean estCoordDonneeCorrect(String chaine) throws BadCoordException {
        try{
            Coord coordonnee = new Coord(chaine);
            return true;
        }catch (BadCoordException e) {
            e.printStackTrace();
        }
        return false;
    }

    //Vérifie qu'il est possible de coder un bateau à partir de ses coordonnées
    public boolean estBateauCorrect(String coPoupe, String coProue) throws BadCoordException {
        ShipCategory categorie = this.getCategory();
        //Vérifie les types et taille bateau :
        if(this.estCoordDonneeCorrect(coPoupe) && this.estCoordDonneeCorrect(coProue)){
            Coord front = new Coord(coProue);
            Coord back = new Coord(coPoupe);
            int taille = Math.abs((front.getX()-back.getX())-(front.getY()-back.getY()))+1;
            if(!(taille > 5) && !(taille < 1)) {
                if(categorie == ShipCategory.AIRCRAFT_CARRIER && taille != 5){
                    return false;
                }
                if(categorie == ShipCategory.BATTLESHIP && taille != 4){
                    return false;
                }
                if(categorie == ShipCategory.CRUISER && taille != 3){
                    return false;
                }
                if(categorie == ShipCategory.DESTROYER && taille != 2){
                    return false;
                }
                if(categorie == ShipCategory.SUBMARINE && taille != 1){
                    return false;
                }

                return true;
            }
        }

        return false;
    }
}
